#!/usr/local/bin/python2.1
"""Manages accounts for pyBlog.

Utilizes cmd to give a simple command-line tool for adding, removing, and
listing accounts.

"""
import cmd, security, ConfigParser, sys, getpass, string

class pyBlogAccts(cmd.Cmd,security.Security):
    """Main class that subclasses cmd.Cmd)."""
    def __init__(self):
        """Sets all global values and opens up security.conf ."""
        self.prompt='pyBlog >>'
        security.Security.__init__(self)

    def do_create_user(self,name):
        """Creates a new user.
        create-user <name> -- Creates a new user, <name>.  Will prompt for
            password.  It will not be echoed.  You will then be prompted to
            enter the password one more time.  If no exceptions are thrown, then
            you will be notified that the creation was successful.

        """
        pass1=getpass.getpass("Enter the password for '%s':" % name)
        pass2=getpass.getpass("Enter the password again for verification:")
        if pass1==pass2:
            if self.createUser(name,pass1):
                print 'Account created successfully'
            else:
                print 'An error occured while trying to create the account'
        del pass1,pass2

    def help_create_user(self):
        print """create_user <name>:
            Prompts for the password for <name> and then creates account.
            Nothing is shown when the password is typed in."""

    def do_remove_user(self,name):
        """Removes a user.
        remove_user <name> -- Removes <name>.  Will prompt again to verify.
            Typing 'yes' will remove the account.

        """
        answer=raw_input('Are you sure you want to remove the user? (yes/no):')
        if string.lower(answer)=='yes':
            try:
                self.conf.remove_option('passwords',name)
                self.conf.remove_option('permissions',name)
                self.saveConf()
                print 'Removal successful.'
            except:
                print 'An error occured while trying to remove the account (might not exist).'

    def help_remove_user(self,crap=None):
        print """remove_user <name>:
             removes account named <name>."""

    def do_list_users(self,crap=None):
        """Lists all user accounts.
        list_users -- prints out all user accounts.

        """
        users=self.conf.options('passwords')
        for user in users:
            print user

    def help_list_users(self):
        print """list_users:
            Prints a list of all usernames."""

    def do_exit(self,crap=None):
        """Exits the program."""
        sys.exit()

    def help_exit(self):
        print """exit:
        Exits the program."""

    def do_basic_help(self,crap=None):
        """Lists all available commands."""
        print """The available commands are:
             create_user
             remove_user
             list_users
             exit
             help

        For help for any command, type '?<command>'."""

    def emptyline(self):
        """Triggers do_basic_help() when an empty line is entered."""
        self.do_basic_help()

    def default(self,line):
        """When an unknown command is enter do_basic_help() is run."""
        print 'Unknown command:',line
        print
        self.do_basic_help()




if __name__=='__main__':
    pyBlogAccts().cmdloop()
