"""Handles archiving for pyBlog using cPickle.

Essentially creates a huge dictionary of all post objects.

"""
from __future__ import nested_scopes
import cPickle, time, shutil, os, re, string
from exceptions import Exception
import strings

class PostDBError(Exception):
	"""Generic exception class."""
	def __init__(self, args):
		self.args=args

class PostDB:
    """Database class."""
    def __init__(self):
	"""Fetches database file location."""
        self.db_location='post_DB'

    def openDB(self):
	"""Opens database file."""
        if not os.path.isfile(self.db_location): return {}
        else:
            DB_FILE=open(self.db_location,'r')
            try: open_DB=cPickle.load(DB_FILE)
            finally: DB_FILE.close()
            return open_DB

    def saveDB(self,new_DB):
	"""Saves database file."""
        if os.path.isfile(self.db_location): shutil.copyfile(self.db_location,self.db_location+'~')
        DB_FILE=open(self.db_location,'w')
        try: cPickle.dump(new_DB,DB_FILE)
        except:
            DB_FILE.close()
            if os.path.isfile(self.db_location+'~'): shutil.copyfile(self.db_location+'~',self.db_location)
            return 0
        else:
            DB_FILE.close()
            if os.path.isfile(self.db_location+'~'): os.remove(self.db_location+'~')
            return 1

    def addPost(self, post_obj):
	"""Adds post to the database."""
        current_date=time.strftime('%Y%m%d',time.gmtime())
        open_DB=self.openDB()
        if not open_DB.has_key(current_date):
                open_DB[current_date]=post_obj
                self.saveDB(open_DB)
                return current_date
        else:
                key_list=open_DB.keys()
                key_list.sort()
                base_key_location=key_list.index(current_date)
                latest_key_RE=re.compile('^%s(?P<suffix>.*)' % current_date)
                for cnt in xrange(base_key_location,len(key_list)):
                        key_compare=latest_key_RE.match(key_list[cnt])
                        if key_compare: continue
                        else:
                                last_location=cnt-1
                                last_suffix=latest_key_RE.match(key_list[last_location]).group('suffix')
                                break
                else:
                        last_location=cnt
                        last_suffix=latest_key_RE.match(key_list[last_location]).group('suffix')
                try:
                        char_location=string.lowercase.index(last_suffix)
                        new_key=current_date+string.lowercase[char_location+1]
                except IndexError:
                        raise PostDBError,'Max posts for the day reached for archive'
                open_DB[new_key]=post_obj
                self.saveDB(open_DB)
                return new_key

    def editPost(self,post_obj,ID):
	"""Replaces post with <ID> with new post object."""
        open_DB=self.openDB()
        try:
            open_DB[ID]=post_obj
            self.saveDB(open_DB)
            return 1
        except: return 0
        

    def delPost(self, ID):
	"""Removes a post."""
        try:
            open_DB=self.openDB()
            del open_DB[ID]
            self.saveDB(open_DB)
            return 1
        except:
            return 0

    def searchPosts(self, post_obj, ID=None):
	"""Searches through all posts for a match to the post object.
	The post object should contain instance values that contain regex expressions
	to be used to search values of the same instance value name in the stored post
	objects."""
        matching_posts=[]
        open_DB=self.openDB()
        if ID: keys=[ID]
        else: keys=open_DB.keys()
        for search_string_key in post_obj.keys():
            try: keys=filter(lambda x: re.search(strings.raw(post_obj[search_string_key]),open_DB[x].data_dict[search_string_key]),keys)
            except KeyError: pass
        return zip(keys,map(lambda x: open_DB[x],keys))
                        
    def fetchPost(self, ID):
        """Return a post based on its ID."""
        open_DB=self.openDB()
        try:
            post=open_DB[ID]
            return post
        except KeyError: return 0
