"""Contains functions that replace functions or fill in gaps in the built-in os module.

FUNCTIONS:
    listdir(path=so.getcwd(), specific=ALL) -> list -- Extends os.listdir() by
                making default path os.getcwd() and allowing you to specify
                whether you want everything, only files, or only directories
                listed.

    touch(filename) -> integer -- Touches the filename just like the UNIX tool.

    split(path) -> tuple -- Takes the path and splits it into directory,
                filename, and file extension.

    chmod(path, octal) -- Takes an octal value and sets the permissions on path.

VARIABLES:
    ALL -- Used by listdir() to specify to return all contents of the directory.
    FILES -- Used by listdir() to specify to return only files.
    DIR -- Used by listdir() to specify to return only directories.

I would like to add a replacement to walk that will recursively go through a
root path and construct a data structure that can then take a function and apply
that function to every file that was found.  Consider using absolute path names
to eliminate need of keeping track of what directory the files were found in.

"""
import os,numbers



ALL='all'
FILES='files'
DIR='dir'



def listdir (path=os.getcwd(), specific=ALL):
    """-> contents of directory [list]
    Takes a path (os.getcwd() by default) and returns its contents based on what
    is specified by the specific parameter.
    
    """
    specific=specific.lower()
    if specific!=ALL and specific!=FILES and specific!=DIR:
        raise NameError
    indir=os.listdir(path)
    newdir=[]
    if specific == FILES:
        for Current in indir:
            if os.path.isfile(os.path.join(path, Current)):
                newdir.append(Current)
    elif specific == DIR:
        for Current in indir:
            if os.path.isdir(os.path.join(path, Current)):
                newdir.append(Current)
    else:
        newdir=indir[:]
    return newdir



def touch(filename):
    """Provides same functionality as touch for UNIX"""
    try:
        open(filename,'w').close()
    except:
        return 0
    else:
        return 1



def split(path):
    """-> (directory, filename, file extension) [tuple]
    Takes in path and returns a tuple containing the directory, filename, and file extension

    """
    split_result=os.path.split(path)
    splitext_result=os.path.splitext(split_result[1])
    return (split_result[0],splitext_result[0],splitext_result[1])



def chmod(path, mode):
    decimal_mode=number.nToDecimal(mode,8)
    os.chmod(path,decimal_mode) 
